@@ -26,7 +26,6 @@
 
 #include "nautilus-application.h"
 
-#include <eel/eel-gtk-extensions.h>
 #include <eel/eel-stock-dialogs.h>
 #include <fcntl.h>
 #include <gdk/gdk.h>
@@ -34,7 +33,7 @@
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
-#include <libhandy-1/handy.h>
+#include <libadwaita-1/adwaita.h>
 #include <nautilus-extension.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -43,6 +42,8 @@
 #include "nautilus-debug.h"
 
 #include "nautilus-bookmark-list.h"
+#include "nautilus-clipboard.h"
+#include "nautilus-dbus-launcher.h"
 #include "nautilus-dbus-manager.h"
 #include "nautilus-directory-private.h"
 #include "nautilus-file.h"
@@ -84,12 +85,13 @@ typedef struct
     NautilusFileUndoManager *undo_manager;
 
     NautilusTagManager *tag_manager;
-    GCancellable *tag_manager_cancellable;
+
+    NautilusDBusLauncher *dbus_launcher;
 
     guint previewer_selection_id;
 } NautilusApplicationPrivate;
 
-G_DEFINE_TYPE_WITH_PRIVATE (NautilusApplication, nautilus_application, GTK_TYPE_APPLICATION);
+G_DEFINE_TYPE_WITH_PRIVATE (NautilusApplication, nautilus_application, ADW_TYPE_APPLICATION);
 
 void
 nautilus_application_set_accelerator (GApplication *app,
@@ -167,7 +169,7 @@ check_required_directories (NautilusApplication *self)
         GSList *l;
         char *error_string;
         g_autofree char *detail_string = NULL;
-        GtkDialog *dialog;
+        AdwMessageDialog *dialog;
 
         ret = FALSE;
 
@@ -240,8 +242,7 @@ menu_provider_init_callback (void)
 }
 
 NautilusWindow *
-nautilus_application_create_window (NautilusApplication *self,
-                                    GdkScreen           *screen)
+nautilus_application_create_window (NautilusApplication *self)
 {
     NautilusWindow *window;
     gboolean maximized;
@@ -252,7 +253,7 @@ nautilus_application_create_window (NautilusApplication *self,
     g_return_val_if_fail (NAUTILUS_IS_APPLICATION (self), NULL);
     nautilus_profile_start (NULL);
 
-    window = nautilus_window_new (screen);
+    window = nautilus_window_new ();
 
     maximized = g_settings_get_boolean
                     (nautilus_window_state, NAUTILUS_WINDOW_STATE_MAXIMIZED);
@@ -369,7 +370,7 @@ real_open_location_full (NautilusApplication *self,
     GFile *old_location = NULL;
     char *old_uri, *new_uri;
     gboolean use_same;
-    GdkScreen *screen;
+    GdkDisplay *display;
 
     use_same = TRUE;
     /* FIXME: We are having problems on getting the current focused window with
@@ -384,7 +385,10 @@ real_open_location_full (NautilusApplication *self,
     {
         active_slot = nautilus_window_get_active_slot (active_window);
         /* Just for debug.*/
-        old_location = nautilus_window_slot_get_location (active_slot);
+        if (active_slot != NULL)
+        {
+            old_location = nautilus_window_slot_get_location (active_slot);
+        }
     }
 
 
@@ -441,12 +445,13 @@ real_open_location_full (NautilusApplication *self,
     }
     else
     {
-        screen = active_window != NULL ?
-                 gtk_window_get_screen (GTK_WINDOW (active_window)) :
-                 gdk_screen_get_default ();
+        display = active_window != NULL ?
+                  gtk_root_get_display (GTK_ROOT (active_window)) :
+                  gdk_display_get_default ();
 
-        target_window = nautilus_application_create_window (self, screen);
+        target_window = nautilus_application_create_window (self);
         /* Whatever the caller says, the slot won't be the same */
+        gtk_window_set_display (GTK_WINDOW (target_window), display);
         target_slot = NULL;
     }
 
@@ -465,7 +470,7 @@ open_window (NautilusApplication *self,
     NautilusWindow *window;
 
     nautilus_profile_start (NULL);
-    window = nautilus_application_create_window (self, gdk_screen_get_default ());
+    window = nautilus_application_create_window (self);
 
     if (location != NULL)
     {
@@ -506,7 +511,7 @@ nautilus_application_open_location (NautilusApplication *self,
 
     if (!slot)
     {
-        window = nautilus_application_create_window (self, gdk_screen_get_default ());
+        window = nautilus_application_create_window (self);
     }
     else
     {
@@ -587,8 +592,7 @@ nautilus_application_finalize (GObject *object)
 
     g_clear_object (&priv->tag_manager);
 
-    g_cancellable_cancel (priv->tag_manager_cancellable);
-    g_clear_object (&priv->tag_manager_cancellable);
+    g_clear_object (&priv->dbus_launcher);
 
     G_OBJECT_CLASS (nautilus_application_parent_class)->finalize (object);
 }
@@ -635,7 +639,7 @@ static int
 do_perform_self_checks (void)
 {
 #ifndef NAUTILUS_OMIT_SELF_CHECK
-    gtk_init (NULL, NULL);
+    gtk_init ();
 
     nautilus_profile_start (NULL);
     /* Run the checks (each twice) for nautilus and libnautilus-private. */
@@ -765,23 +769,19 @@ action_help (GSimpleAction *action,
     GError *error = NULL;
 
     window = gtk_application_get_active_window (application);
-    gtk_show_uri_on_window (window, "help:gnome-help/files",
-                            gtk_get_current_event_time (), &error);
+    gtk_show_uri (window, "help:gnome-help/files", GDK_CURRENT_TIME);
 
     if (error)
     {
-        dialog = gtk_message_dialog_new (window ? GTK_WINDOW (window) : NULL,
-                                         GTK_DIALOG_MODAL,
-                                         GTK_MESSAGE_ERROR,
-                                         GTK_BUTTONS_OK,
-                                         _("There was an error displaying help: \n%s"),
-                                         error->message);
-        g_signal_connect (G_OBJECT (dialog), "response",
-                          G_CALLBACK (gtk_widget_destroy),
-                          NULL);
+        dialog = adw_message_dialog_new (window ? GTK_WINDOW (window) : NULL,
+                                         NULL, NULL);
+        adw_message_dialog_format_heading (ADW_MESSAGE_DIALOG (dialog),
+                                           _("There was an error displaying help: \n%s"),
+                                           error->message);
+        adw_message_dialog_add_response (ADW_MESSAGE_DIALOG (dialog), "ok", _("_OK"));
+        adw_message_dialog_set_default_response (ADW_MESSAGE_DIALOG (dialog), "ok");
 
-        gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-        gtk_widget_show (dialog);
+        gtk_window_present (GTK_WINDOW (dialog));
         g_error_free (error);
     }
 }
@@ -791,7 +791,7 @@ action_kill (GSimpleAction *action,
              GVariant      *parameter,
              gpointer       user_data)
 {
-    GtkApplication *application = user_data;
+    AdwApplication *application = user_data;
 
     /* we have been asked to force quit */
     g_application_quit (G_APPLICATION (application));
@@ -817,29 +817,6 @@ action_quit (GSimpleAction *action,
     g_list_free (windows);
 }
 
-static void
-action_show_hide_sidebar (GObject    *object,
-                          GParamSpec *pspec,
-                          gpointer   *user_data)
-{
-    GList *window, *windows;
-    GVariant *state = g_action_get_state (G_ACTION (object));
-
-    windows = gtk_application_get_windows (GTK_APPLICATION (user_data));
-
-    for (window = windows; window != NULL; window = window->next)
-    {
-        if (g_variant_get_boolean (state))
-        {
-            nautilus_window_show_sidebar (window->data);
-        }
-        else
-        {
-            nautilus_window_hide_sidebar (window->data);
-        }
-    }
-}
-
 static void
 action_show_help_overlay (GSimpleAction *action,
                           GVariant      *state,
@@ -851,29 +828,11 @@ action_show_help_overlay (GSimpleAction *action,
     g_action_group_activate_action (G_ACTION_GROUP (window), "show-help-overlay", NULL);
 }
 
-static gboolean
-variant_get_mapping (GValue   *value,
-                     GVariant *variant,
-                     gpointer  user_data)
-{
-    g_value_set_variant (value, variant);
-    return TRUE;
-}
-
-static GVariant *
-variant_set_mapping (const GValue       *value,
-                     const GVariantType *expected_type,
-                     gpointer            user_data)
-{
-    return g_value_get_variant (value);
-}
-
 const static GActionEntry app_entries[] =
 {
     { "new-window", action_new_window, NULL, NULL, NULL },
     { "clone-window", action_clone_window, NULL, NULL, NULL },
     { "preferences", action_preferences, NULL, NULL, NULL },
-    { "show-hide-sidebar", NULL, NULL, "true", NULL },
     { "about", action_about, NULL, NULL, NULL },
     { "help", action_help, NULL, NULL, NULL },
     { "quit", action_quit, NULL, NULL, NULL },
@@ -884,36 +843,17 @@ const static GActionEntry app_entries[] =
 static void
 nautilus_init_application_actions (NautilusApplication *app)
 {
-    GAction *sidebar_action;
-
     g_action_map_add_action_entries (G_ACTION_MAP (app),
                                      app_entries, G_N_ELEMENTS (app_entries),
                                      app);
 
 
-    sidebar_action = g_action_map_lookup_action (G_ACTION_MAP (app),
-                                                 "show-hide-sidebar");
-    g_signal_connect (sidebar_action,
-                      "notify::state",
-                      G_CALLBACK (action_show_hide_sidebar),
-                      app);
-    g_settings_bind_with_mapping (nautilus_window_state,
-                                  NAUTILUS_WINDOW_STATE_START_WITH_SIDEBAR,
-                                  sidebar_action,
-                                  "state",
-                                  G_SETTINGS_BIND_DEFAULT,
-                                  variant_get_mapping,
-                                  variant_set_mapping,
-                                  NULL, NULL);
-
     nautilus_application_set_accelerator (G_APPLICATION (app),
                                           "app.clone-window", "<Primary>n");
     nautilus_application_set_accelerator (G_APPLICATION (app),
                                           "app.help", "F1");
     nautilus_application_set_accelerator (G_APPLICATION (app),
                                           "app.quit", "<Primary>q");
-    nautilus_application_set_accelerator (G_APPLICATION (app),
-                                          "app.show-hide-sidebar", "F9");
 }
 
 static void
@@ -1102,86 +1042,16 @@ nautilus_application_init (NautilusApplication *self)
                                                  NULL);
 
     priv->undo_manager = nautilus_file_undo_manager_new ();
+    priv->tag_manager = nautilus_tag_manager_new ();
 
-    priv->tag_manager_cancellable = g_cancellable_new ();
-    priv->tag_manager = nautilus_tag_manager_get ();
-    nautilus_tag_manager_set_cancellable (priv->tag_manager,
-                                          priv->tag_manager_cancellable);
+    priv->dbus_launcher = nautilus_dbus_launcher_new ();
 
     g_application_add_main_option_entries (G_APPLICATION (self), options);
 
     nautilus_ensure_extension_points ();
     nautilus_ensure_extension_builtins ();
-}
 
-static void
-theme_changed (GtkSettings *settings)
-{
-    static GtkCssProvider *provider = NULL;
-    static GtkCssProvider *permanent_provider = NULL;
-    gchar *theme;
-    GdkScreen *screen;
-    GFile *file;
-
-    g_object_get (settings, "gtk-theme-name", &theme, NULL);
-    screen = gdk_screen_get_default ();
-
-    /* CSS that themes can override */
-    if (g_str_equal (theme, "Adwaita") ||
-        g_str_equal (theme, "Adwaita-dark") ||
-        g_str_equal (theme, "HighContrast") ||
-        g_str_equal (theme, "HighContrastInverse"))
-    {
-        if (provider == NULL)
-        {
-            provider = gtk_css_provider_new ();
-            file = g_file_new_for_uri ("resource:///org/gnome/nautilus/css/Adwaita.css");
-            gtk_css_provider_load_from_file (provider, file, NULL);
-            g_object_unref (file);
-        }
-
-        gtk_style_context_add_provider_for_screen (screen,
-                                                   GTK_STYLE_PROVIDER (provider),
-                                                   GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
-    }
-    else if (provider != NULL)
-    {
-        gtk_style_context_remove_provider_for_screen (screen,
-                                                      GTK_STYLE_PROVIDER (provider));
-        g_clear_object (&provider);
-    }
-
-    /* CSS we want to always load for any theme */
-    if (permanent_provider == NULL)
-    {
-        permanent_provider = gtk_css_provider_new ();
-        file = g_file_new_for_uri ("resource:///org/gnome/nautilus/css/nautilus.css");
-        gtk_css_provider_load_from_file (permanent_provider, file, NULL);
-        /* The behavior of two style providers with the same priority is
-         * undefined and gtk happens to prefer the provider that got added last.
-         * Use a higher priority here to avoid this problem.
-         */
-        gtk_style_context_add_provider_for_screen (screen,
-                                                   GTK_STYLE_PROVIDER (permanent_provider),
-                                                   GTK_STYLE_PROVIDER_PRIORITY_APPLICATION + 1);
-        g_object_unref (file);
-    }
-
-    g_free (theme);
-}
-
-static void
-setup_theme_extensions (void)
-{
-    GtkSettings *settings;
-
-    /* Set up a handler to load our custom css for Adwaita.
-     * See https://bugzilla.gnome.org/show_bug.cgi?id=732959
-     * for a more automatic solution that is still under discussion.
-     */
-    settings = gtk_settings_get_default ();
-    g_signal_connect (settings, "notify::gtk-theme-name", G_CALLBACK (theme_changed), NULL);
-    theme_changed (settings);
+    nautilus_clipboard_register ();
 }
 
 NautilusApplication *
@@ -1276,15 +1146,8 @@ nautilus_application_startup_common (NautilusApplication *self)
      */
     G_APPLICATION_CLASS (nautilus_application_parent_class)->startup (G_APPLICATION (self));
 
-    hdy_init ();
-
-    hdy_style_manager_set_color_scheme (hdy_style_manager_get_default (),
-                                        HDY_COLOR_SCHEME_PREFER_LIGHT);
-
     gtk_window_set_default_icon_name (APPLICATION_ID);
 
-    setup_theme_extensions ();
-
     /* initialize preferences and create the global GSettings objects */
     nautilus_global_preferences_init ();
 
@@ -1306,13 +1169,16 @@ nautilus_application_startup_common (NautilusApplication *self)
 
     nautilus_init_application_actions (self);
 
-    nautilus_tag_manager_maybe_migrate_tracker2_data (priv->tag_manager);
+    if (g_strcmp0 (g_getenv ("RUNNING_TESTS"), "TRUE") != 0)
+    {
+        nautilus_tag_manager_maybe_migrate_tracker2_data (priv->tag_manager);
+    }
 
     nautilus_profile_end (NULL);
 
     g_signal_connect (self, "shutdown", G_CALLBACK (on_application_shutdown), NULL);
 
-    g_signal_connect_object (gtk_icon_theme_get_default (),
+    g_signal_connect_object (gtk_icon_theme_get_for_display (gdk_display_get_default ()),
                              "changed",
                              G_CALLBACK (icon_theme_changed_callback),
                              NULL, 0);
@@ -1623,3 +1489,18 @@ nautilus_application_search (NautilusApplication *self,
     window = open_window (self, location);
     nautilus_window_search (window, query);
 }
+
+gboolean
+nautilus_application_is_sandboxed (void)
+{
+    static gboolean ret;
+
+    static gsize init = 0;
+    if (g_once_init_enter (&init))
+    {
+        ret = g_file_test ("/.flatpak-info", G_FILE_TEST_EXISTS);
+        g_once_init_leave (&init, 1);
+    }
+
+    return ret;
+}
