@@ -28,11 +28,10 @@ G_BEGIN_DECLS
 
 G_DECLARE_FINAL_TYPE (NautilusTagManager, nautilus_tag_manager, NAUTILUS, TAG_MANAGER, GObject);
 
+NautilusTagManager* nautilus_tag_manager_new                (void);
+NautilusTagManager* nautilus_tag_manager_new_dummy          (void);
 NautilusTagManager* nautilus_tag_manager_get                (void);
 
-void                nautilus_tag_manager_set_cancellable    (NautilusTagManager *tag_manager,
-                                                             GCancellable *cancellable);
-
 GList*              nautilus_tag_manager_get_starred_files (NautilusTagManager *self);
 
 void                nautilus_tag_manager_star_files         (NautilusTagManager  *self,
