@@ -1,5 +1,5 @@
 #include "nautilus-view-model.h"
-#include "nautilus-view-item-model.h"
+#include "nautilus-view-item.h"
 #include "nautilus-global-preferences.h"
 
 struct _NautilusViewModel
@@ -8,13 +8,15 @@ struct _NautilusViewModel
 
     GHashTable *map_files_to_model;
     GListStore *internal_model;
-    NautilusViewModelSortData *sort_data;
+    GtkMultiSelection *selection_model;
+    GtkSorter *sorter;
+    gulong sorter_changed_id;
 };
 
 static GType
 nautilus_view_model_get_item_type (GListModel *list)
 {
-    return NAUTILUS_TYPE_VIEW_ITEM_MODEL;
+    return NAUTILUS_TYPE_VIEW_ITEM;
 }
 
 static guint
@@ -52,22 +54,89 @@ nautilus_view_model_list_model_init (GListModelInterface *iface)
     iface->get_item = nautilus_view_model_get_item;
 }
 
+
+static gboolean
+nautilus_view_model_is_selected (GtkSelectionModel *model,
+                                 guint              position)
+{
+    NautilusViewModel *self = NAUTILUS_VIEW_MODEL (model);
+    GtkSelectionModel *selection_model = GTK_SELECTION_MODEL (self->selection_model);
+
+    return gtk_selection_model_is_selected (selection_model, position);
+}
+
+static GtkBitset *
+nautilus_view_model_get_selection_in_range (GtkSelectionModel *model,
+                                            guint              pos,
+                                            guint              n_items)
+{
+    NautilusViewModel *self = NAUTILUS_VIEW_MODEL (model);
+    GtkSelectionModel *selection_model = GTK_SELECTION_MODEL (self->selection_model);
+
+    return gtk_selection_model_get_selection_in_range (selection_model, pos, n_items);
+}
+
+static gboolean
+nautilus_view_model_set_selection (GtkSelectionModel *model,
+                                   GtkBitset         *selected,
+                                   GtkBitset         *mask)
+{
+    NautilusViewModel *self = NAUTILUS_VIEW_MODEL (model);
+    GtkSelectionModel *selection_model = GTK_SELECTION_MODEL (self->selection_model);
+    gboolean res;
+
+    res = gtk_selection_model_set_selection (selection_model, selected, mask);
+    if (res)
+    {
+        guint min = gtk_bitset_get_minimum (mask);
+        guint max = gtk_bitset_get_maximum (mask);
+
+        if (min <= max)
+        {
+            gtk_selection_model_selection_changed (selection_model, min, max - min + 1);
+        }
+    }
+    return res;
+}
+
+
+static void
+nautilus_view_model_selection_model_init (GtkSelectionModelInterface *iface)
+{
+    iface->is_selected = nautilus_view_model_is_selected;
+    iface->get_selection_in_range = nautilus_view_model_get_selection_in_range;
+    iface->set_selection = nautilus_view_model_set_selection;
+}
+
 G_DEFINE_TYPE_WITH_CODE (NautilusViewModel, nautilus_view_model, G_TYPE_OBJECT,
                          G_IMPLEMENT_INTERFACE (G_TYPE_LIST_MODEL,
-                                                nautilus_view_model_list_model_init))
+                                                nautilus_view_model_list_model_init)
+                         G_IMPLEMENT_INTERFACE (GTK_TYPE_SELECTION_MODEL,
+                                                nautilus_view_model_selection_model_init))
 
 enum
 {
     PROP_0,
-    PROP_SORT_TYPE,
+    PROP_SORTER,
     N_PROPS
 };
 
+static GParamSpec *properties[N_PROPS] = { NULL, };
+
 static void
 dispose (GObject *object)
 {
     NautilusViewModel *self = NAUTILUS_VIEW_MODEL (object);
 
+    if (self->selection_model != NULL)
+    {
+        g_signal_handlers_disconnect_by_func (self->selection_model,
+                                              gtk_selection_model_selection_changed,
+                                              self);
+        g_object_unref (self->selection_model);
+        self->selection_model = NULL;
+    }
+
     if (self->internal_model != NULL)
     {
         g_signal_handlers_disconnect_by_func (self->internal_model,
@@ -77,6 +146,8 @@ dispose (GObject *object)
         self->internal_model = NULL;
     }
 
+    g_clear_signal_handler (&self->sorter_changed_id, self->sorter);
+
     G_OBJECT_CLASS (nautilus_view_model_parent_class)->dispose (object);
 }
 
@@ -88,10 +159,7 @@ finalize (GObject *object)
     G_OBJECT_CLASS (nautilus_view_model_parent_class)->finalize (object);
 
     g_hash_table_destroy (self->map_files_to_model);
-    if (self->sort_data)
-    {
-        g_free (self->sort_data);
-    }
+    g_clear_object (&self->sorter);
 }
 
 static void
@@ -104,9 +172,9 @@ get_property (GObject    *object,
 
     switch (prop_id)
     {
-        case PROP_SORT_TYPE:
+        case PROP_SORTER:
         {
-            g_value_set_object (value, self->sort_data);
+            g_value_set_object (value, nautilus_view_model_get_sorter (self));
         }
         break;
 
@@ -127,9 +195,9 @@ set_property (GObject      *object,
 
     switch (prop_id)
     {
-        case PROP_SORT_TYPE:
+        case PROP_SORTER:
         {
-            nautilus_view_model_set_sort_type (self, g_value_get_object (value));
+            nautilus_view_model_set_sorter (self, g_value_get_object (value));
         }
         break;
 
@@ -147,11 +215,14 @@ constructed (GObject *object)
 
     G_OBJECT_CLASS (nautilus_view_model_parent_class)->constructed (object);
 
-    self->internal_model = g_list_store_new (NAUTILUS_TYPE_VIEW_ITEM_MODEL);
+    self->internal_model = g_list_store_new (NAUTILUS_TYPE_VIEW_ITEM);
+    self->selection_model = gtk_multi_selection_new (g_object_ref (G_LIST_MODEL (self->internal_model)));
     self->map_files_to_model = g_hash_table_new (NULL, NULL);
 
     g_signal_connect_swapped (self->internal_model, "items-changed",
                               G_CALLBACK (g_list_model_items_changed), self);
+    g_signal_connect_swapped (self->selection_model, "selection-changed",
+                              G_CALLBACK (gtk_selection_model_selection_changed), self);
 }
 
 static void
@@ -164,6 +235,14 @@ nautilus_view_model_class_init (NautilusViewModelClass *klass)
     object_class->get_property = get_property;
     object_class->set_property = set_property;
     object_class->constructed = constructed;
+
+    properties[PROP_SORTER] =
+        g_param_spec_object ("sorter",
+                             "", "",
+                             GTK_TYPE_SORTER,
+                             G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
+
+    g_object_class_install_properties (object_class, N_PROPS, properties);
 }
 
 static void
@@ -177,45 +256,54 @@ compare_data_func (gconstpointer a,
                    gpointer      user_data)
 {
     NautilusViewModel *self = NAUTILUS_VIEW_MODEL (user_data);
-    NautilusFile *file_a;
-    NautilusFile *file_b;
 
-    file_a = nautilus_view_item_model_get_file (NAUTILUS_VIEW_ITEM_MODEL ((gpointer) a));
-    file_b = nautilus_view_item_model_get_file (NAUTILUS_VIEW_ITEM_MODEL ((gpointer) b));
-
-    return nautilus_file_compare_for_sort (file_a, file_b,
-                                           self->sort_data->sort_type,
-                                           self->sort_data->directories_first,
-                                           self->sort_data->reversed);
-}
-
-NautilusViewModel *
-nautilus_view_model_new ()
-{
-    return g_object_new (NAUTILUS_TYPE_VIEW_MODEL, NULL);
-}
-
-void
-nautilus_view_model_set_sort_type (NautilusViewModel         *self,
-                                   NautilusViewModelSortData *sort_data)
-{
-    if (self->sort_data)
+    if (self->sorter == NULL)
     {
-        g_free (self->sort_data);
+        return GTK_ORDERING_EQUAL;
     }
 
-    self->sort_data = g_new (NautilusViewModelSortData, 1);
-    self->sort_data->sort_type = sort_data->sort_type;
-    self->sort_data->reversed = sort_data->reversed;
-    self->sort_data->directories_first = sort_data->directories_first;
+    return gtk_sorter_compare (self->sorter, (gpointer) a, (gpointer) b);
+}
+
+static void
+on_sorter_changed (GtkSorter       *sorter,
+                   GtkSorterChange  change,
+                   gpointer         user_data)
+{
+    NautilusViewModel *self = NAUTILUS_VIEW_MODEL (user_data);
 
     g_list_store_sort (self->internal_model, compare_data_func, self);
 }
 
-NautilusViewModelSortData *
-nautilus_view_model_get_sort_type (NautilusViewModel *self)
+NautilusViewModel *
+nautilus_view_model_new (void)
 {
-    return self->sort_data;
+    return g_object_new (NAUTILUS_TYPE_VIEW_MODEL, NULL);
+}
+
+GtkSorter *
+nautilus_view_model_get_sorter (NautilusViewModel *self)
+{
+    return self->sorter;
+}
+
+void
+nautilus_view_model_set_sorter (NautilusViewModel *self,
+                                GtkSorter         *sorter)
+{
+    if (self->sorter != NULL)
+    {
+        g_clear_signal_handler (&self->sorter_changed_id, self->sorter);
+    }
+
+    if (g_set_object (&self->sorter, sorter))
+    {
+        g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_SORTER]);
+    }
+
+    self->sorter_changed_id = g_signal_connect (self->sorter, "changed",
+                                                G_CALLBACK (on_sorter_changed), self);
+    g_list_store_sort (self->internal_model, compare_data_func, self);
 }
 
 GQueue *
@@ -223,39 +311,39 @@ nautilus_view_model_get_items_from_files (NautilusViewModel *self,
                                           GQueue            *files)
 {
     GList *l;
-    NautilusViewItemModel *item_model;
-    GQueue *item_models;
+    guint n_items;
+    GQueue *items;
 
-    item_models = g_queue_new ();
+    n_items = g_list_model_get_n_items (G_LIST_MODEL (self->internal_model));
+    items = g_queue_new ();
     for (l = g_queue_peek_head_link (files); l != NULL; l = l->next)
     {
         NautilusFile *file1;
-        gint i = 0;
 
         file1 = NAUTILUS_FILE (l->data);
-        while ((item_model = g_list_model_get_item (G_LIST_MODEL (self->internal_model), i)))
+        for (guint i = 0; i < n_items; i++)
         {
+            g_autoptr (NautilusViewItem) item = NULL;
             NautilusFile *file2;
             g_autofree gchar *file1_uri = NULL;
             g_autofree gchar *file2_uri = NULL;
 
-            file2 = nautilus_view_item_model_get_file (item_model);
+            item = g_list_model_get_item (G_LIST_MODEL (self->internal_model), i);
+            file2 = nautilus_view_item_get_file (item);
             file1_uri = nautilus_file_get_uri (file1);
             file2_uri = nautilus_file_get_uri (file2);
             if (g_strcmp0 (file1_uri, file2_uri) == 0)
             {
-                g_queue_push_tail (item_models, item_model);
+                g_queue_push_tail (items, item);
                 break;
             }
-
-            i++;
         }
     }
 
-    return item_models;
+    return items;
 }
 
-NautilusViewItemModel *
+NautilusViewItem *
 nautilus_view_model_get_item_from_file (NautilusViewModel *self,
                                         NautilusFile      *file)
 {
@@ -263,8 +351,8 @@ nautilus_view_model_get_item_from_file (NautilusViewModel *self,
 }
 
 void
-nautilus_view_model_remove_item (NautilusViewModel     *self,
-                                 NautilusViewItemModel *item)
+nautilus_view_model_remove_item (NautilusViewModel *self,
+                                 NautilusViewItem  *item)
 {
     guint i;
 
@@ -272,7 +360,7 @@ nautilus_view_model_remove_item (NautilusViewModel     *self,
     {
         NautilusFile *file;
 
-        file = nautilus_view_item_model_get_file (item);
+        file = nautilus_view_item_get_file (item);
         g_list_store_remove (self->internal_model, i);
         g_hash_table_remove (self->map_files_to_model, file);
     }
@@ -286,11 +374,11 @@ nautilus_view_model_remove_all_items (NautilusViewModel *self)
 }
 
 void
-nautilus_view_model_add_item (NautilusViewModel     *self,
-                              NautilusViewItemModel *item)
+nautilus_view_model_add_item (NautilusViewModel *self,
+                              NautilusViewItem  *item)
 {
     g_hash_table_insert (self->map_files_to_model,
-                         nautilus_view_item_model_get_file (item),
+                         nautilus_view_item_get_file (item),
                          item);
     g_list_store_insert_sorted (self->internal_model, item, compare_data_func, self);
 }
@@ -304,13 +392,13 @@ nautilus_view_model_add_items (NautilusViewModel *self,
     int i = 0;
 
     array = g_malloc_n (g_queue_get_length (items),
-                        sizeof (NautilusViewItemModel *));
+                        sizeof (NautilusViewItem *));
 
     for (l = g_queue_peek_head_link (items); l != NULL; l = l->next)
     {
         array[i] = l->data;
         g_hash_table_insert (self->map_files_to_model,
-                             nautilus_view_item_model_get_file (l->data),
+                             nautilus_view_item_get_file (l->data),
                              l->data);
         i++;
     }
@@ -321,3 +409,16 @@ nautilus_view_model_add_items (NautilusViewModel *self,
 
     g_list_store_sort (self->internal_model, compare_data_func, self);
 }
+
+guint
+nautilus_view_model_get_index (NautilusViewModel *self,
+                               NautilusViewItem  *item)
+{
+    guint i = G_MAXUINT;
+    gboolean found;
+
+    found = g_list_store_find (self->internal_model, item, &i);
+    g_warn_if_fail (found);
+
+    return i;
+}
